'This script accompanies the book Windows Management Instrumentation
'by Matthew Lavy and Ashley Meggitt (New Riders, 2001)
'The code is copyright 2001, Matthew Lavy & Ashley Meggitt
'You are free to use and modify the script at will
'provided you understand that all code here is made available AS IS, with
'NO WARRANTY WHATSOEVER. The authors take NO RESPONSIBILITY AT ALL for
'problems that result from the use of this script or any part thereof

'archivelogs.vbs - make a backup and clear all Event Logs
'on a command-line specified computer. Back up to a command-line
'specified archive directory. Optionally only back up if a cmd-line
'specified threshold has been reached
Option Explicit
On Error Resume Next
Dim numThreshold
Dim strMachine
Dim strArchivePath
Dim strMoniker
Dim refWMI
Dim colEventLogs
Dim refEventLog

'how many cmd-line args?
If WScript.Arguments.Count < 2 Then
	WScript.Echo _
		"Usage: archivelogs.vbs <machine> <archive_path> [threshold]"
	WScript.Quit
End If

If WScript.Arguments.Count = 2 Then
	numThreshold = 0
Else
	'ensure that third arg is numeric and set threshold
	numThreshold = WScript.Arguments(2)
	If Not IsNumeric(numThreshold) Then
		WScript.Echo "The third parameter must be a number!"
		WScript.Quit
	End If

	If numThreshold < 0 OR numThreshold > 100 Then
		WScript.Echo "The third parameter must be in the range 0-100"
		WScript.Quit
	End If
End If

'save machine name and archive path
strMachine = WScript.Arguments(0)
strArchivePath = WScript.Arguments(1)

'attempt to connect to WMI
strMoniker = "winMgmts:{(Backup,Security)}!\\" & strMachine
Set refWMI = GetObject(strMoniker)
If Err <> 0 Then
	WScript.Echo "Could not connect to the WMI service."
	WScript.Quit
End If

'get a collection of Win32_NTEventLogFile objects
Set colEventLogs = refWMI.InstancesOf("Win32_NTEventLogFile")
If Err <> 0 Then
	WScript.Echo "Could not retrieve Event Log objects"
	WScript.Quit
End If

'iterate through each log
For Each refEventLog In colEventLogs
	'if shouldAct() returns non-zero attempt to back up
	If shouldAct(refEventLog.FileSize,refEventLog.MaxFileSize) <> 0 Then
		If refEventLog.ClearEventLog( _
			makeFileName(refEventLog.LogfileName)) = 0 Then
				WScript.Echo refEventLog.LogfileName & _
					" archived successfully"
		Else
			WScript.Echo refEventLog.LogfileName & _
				" could not be archived"
		End If
	Else
		WScript.Echo refEventLog.LogfileName & _
			" has not exceeded the backup level"
	End If
Next
Set refEventLog = Nothing
Set colEventLogs = Nothing
Set refWMI = Nothing

'--- end of script

'shouldAct() calculates whether the backup and clear
'should be performed based on cursize, maxsize and
'threshold. Returns 0 for "do not act", non-zero otherwise
Function shouldAct(numCurSize, numMaxSize)
	If (numCurSize/numMaxSize)*100 > numThreshold Then
		shouldAct = 1
	Else
		shouldAct = 0
	End If
End Function

'makeFileName() generates a string that can be used as an
'archive file name from the machine name, log name and date
Function makeFileName(strLogname)
	makeFileName = strArchivePath & "\" & _
		strMachine & "-" & strLogname & "-" & _
		Year(Now) & Month(Now) & Day(Now) & ".evt"
End Function
