'This script accompanies the book Windows Management Instrumentation
'by Matthew Lavy and Ashley Meggitt (New Riders, 2001)
'The code is copyright 2001, Matthew Lavy & Ashley Meggitt
'You are free to use and modify the script at will
'provided you understand that all code here is made available AS IS, with
'NO WARRANTY WHATSOEVER. The authors take NO RESPONSIBILITY AT ALL for
'problems that result from the use of this script or any part thereof

'showperms.vbs - display all security information about a named
'file or directory
Option Explicit
On Error Resume Next
Dim refSetting
Dim refSD
Dim refItem

'check that arguments are valid
If WScript.arguments.count <> 1 Then
	WScript.Echo "Usage: showperms.vbs <object>"
	WScript.quit
End If

'attempt to retrieve a Win32_LogicalFileSecuritySetting for
'the file or directory specified on the command line
Set refSetting = GetObject("winMgmts:{(Security,Restore)}" & _
	"Win32_LogicalFileSecuritySetting='" & _
WScript.arguments.item(0) & "'")

If Err <> 0 Then
	WScript.Echo "Cannot get security for " & WScript.arguments.item(0)
	WScript.quit
End If

'output owner name
If refSetting.GetSecurityDescriptor(refSD) = 0 Then
	WScript.Echo refSetting.Path & " owned by " & refSD.Owner.Name
	'loop through the DACL - for each Trustee, output the name and a decoded
	'textual representation of the AccessMask and AceFlags
	For Each refItem In refSD.DACL
		WScript.Echo " Access Control Entry for " & refItem.Trustee.Name
		If refItem.AceType = 0 Then
			WScript.Echo " The following rights are conferred:-"
		Else
			WScript.Echo " The following rights are denied:-"
		End If
		WScript.Echo " " & decodeAccessMask(refItem.AccessMask)
		WScript.Echo " ACE inheritance controlled as follows:-"
		WScript.Echo " " & decodeAceFlags(refItem.AceFlags)
	Next

	'now do the same for the SACL
	For Each refItem In refSD.SACL
		If Err <> 0 Then
			'there is no SACL
			WScript.Echo "SACL is NULL: Nothing will be audited"
		Else
			Wscript.Echo " The following actions will be audited for " & _
			refItem.Trustee.Name
			WScript.Echo " " & decodeAccessMask(refItem.AccessMask)
			WScript.Echo " ACE inheritance controlled as follows:-"
			WScript.Echo " " & decodeAceFlags(refItem.AceFlags)
		End If
	Next
	Set refSD = Nothing
End If

Set refSetting = Nothing
'decodeAccessMask() turns an AccessMask bitmask into a textual
'representation
Function decodeAccessMask(m)
	Dim s
	If m AND 1048576 Then s = s + "Synchronize,"
	If m AND 524288 Then s = s + "Write owner,"
	If m AND 262144 Then s = s + "Write ACL,"
	If m AND 131072 Then s = s + "Read Security,"
	If m AND 65536 Then s = s + "Delete,"
	If m AND 256 Then s = s + "Write Attr,"
	If m AND 128 Then s = s + "Read Attr,"
	If m AND 64 Then s = s + "Delete Dir,"
	If m AND 32 Then s = s + "Execute,"
	If m AND 16 Then s = s + "Write ExtAttr,"
	If m AND 8 Then s = s + "Read ExtAttr," If m AND 4 Then s = s + "Append,"
	If m AND 2 Then s = s + "Write," If m AND 1 Then s = s + "Read."
	decodeAccessMask = s
End Function

'decodeAceFlags() turns an AceFlags bitmask into a textual
'representation
Function decodeAceFlags(m)
	Dim s
	If m AND 16 Then s = s + "Has been inherited,"
	If m AND 8 Then s = s + "Not effective will be inherited,"
	If m AND 4 Then s = s + "Children will not pass on,"
	If m AND 2 Then s = s + "Containers will inherit and pass on,"
	If m AND 1 Then _
	s = s + "Non-containers will inherit and pass on."
	decodeAceFlags = s
End Function
